# Data and code for Dyadic Physical Activity Planning with a Virtual Coach: Using Reinforcement Learning to Select Persuasive Strategies

**Author:** Andrei Stefan <br>
**Date:** November 2023

This repository is meant to show and reproduce our analyses for the paper "Dyadic Physical Activity Planning with a Virtual Coach: Using Reinforcement Learning to Select Persuasive Strategies."
## Contents of this Repository

```
|   ChecklistDataRepositoryReview.pdf
│   README.md
│
└───data
|   └───anonymised_data
|       |   anonymised_data_contemplating.csv
|       |   anonymised_data_conversation_items.csv
|       |   anonymised_data_demographic.csv
|       |   anonymised_data_final.csv
|       |   anonymised_data_final_without_time_and_id.csv
|       |   anonymised_data_high_attitude.csv
|       |   anonymised_data_high_confidence.csv
|       |   anonymised_data_high_perceived_usefulness.csv
|       |   anonymised_data_low_attitude.csv
|       |   anonymised_data_low_confidence.csv
|       |   anonymised_data_low_perceived_usefulness.csv
|       |   anonymised_data_man.csv
|       |   anonymised_data_moderate.csv
|       |   anonymised_data_post_questionnaire.csv
|       |   anonymised_data_preparing.csv
|       |   anonymised_data_prescreening.csv
|       |   anonymised_data_prescreening_no_changes.csv
|       |   anonymised_data_sedentary.csv
|       |   anonymised_data_state_action_state.csv
|       |   anonymised_data_woman_and_non_binary.csv
|   |   c_after.csv
│   │   c_before.csv
│   │   c_changes_to_plan_1_after.csv
│   │   c_changes_to_plan_1_before.csv
│   │   c_changes_to_plan_2_after.csv
│   │   c_changes_to_plan_2_before.csv
│   │   c_deal_with_barriers_after.csv
│   │   c_deal_with_barriers_before.csv
│   │   c_explain_planning_after.csv
│   │   c_explain_planning_before.csv
│   │   c_identify_barriers_after.csv
│   │   c_identify_barriers_before.csv
│   │   c_show_testimonials_after.csv
│   │   c_show_testimonials_before.csv
│   │   constructsMean_14agents.csv (once downloaded)
│   │   data_adjusted.csv
│   │   data_no_features.csv
|   |   prolific_ids.csv
|   |   pu_after.csv
│   │   pu_before.csv
│   │   pu_changes_to_plan_1_after.csv
│   │   pu_changes_to_plan_1_before.csv
│   │   pu_changes_to_plan_2_after.csv
│   │   pu_changes_to_plan_2_before.csv
│   │   pu_deal_with_barriers_after.csv
│   │   pu_deal_with_barriers_before.csv
│   │   pu_explain_planning_after.csv
│   │   pu_explain_planning_before.csv
│   │   pu_identify_barriers_after.csv
│   │   pu_identify_barriers_before.csv
│   │   pu_show_testimonials_after.csv
│   │   pu_show_testimonials_before.csv
|   |	README.md
│   │   thematic_analysis_demotivating.csv
│   │   thematic_analysis_demotivating_examples_for_second_coder.xlsx
│   │   thematic_analysis_demotivating_full.csv
│   │   thematic_analysis_final_double_coding.xlsx
│   │   thematic_analysis_motivating.csv
│   │   thematic_analysis_motivating_examples_for_second_coder.xlsx
│   │   thematic_analysis_motivating_full.csv
|
└───analysis
|    └───exploratory_analysis
|        └───asa questionnaire
|            │   asa_questionnaire.ipynb
|        └───effects_of_actions
|            │   bayesian_t_test_confidence.pdf
|            │   bayesian_t_test_confidence.Rmd
|            │   bayesian_t_test_perceived_usefulness.pdf
|            │   bayesian_t_test_perceived_usefulness.Rmd
|            │   bayesian_t_test_start_end.pdf
|            │   bayesian_t_test_start_end.Rmd
|            │   Dockerfile
|            │   effects_of_actions.ipynb
|            │   JAGS-4.3.0.tar.gz
|        └───planning habits and enjoyment
|            │   planning_habits_and_enjoyment.ipynb
|        └───thematic analysis
|            │   thematic_analysis.ipynb
|    └───main analysis questions
|        └───files_for_g
|        └───state_files
│            │   all_possible_states.txt
│            │   states_for_g.txt
│            │   states_for_q4.txt
│            │   states_for_q6.txt
│        │   analysis_2_3.ipynb
│        │   analysis_4_5_6.ipynb
│        │   g_algorithm.ipynb
│   │   demographic.ipynb
│   │   Dockerfile
│   │   preprocess.ipynb
|
└───results
│   │   analysis_2_3.html
│   │   analysis_4_5_6.html
│   │   g_algorithm.html
│   │   preprocess.html
│   │   demographic.html
│   │   asa_questionnaire.html
│   │   effects_of_actions.html
│   │   planning_habits_and_enjoyment.html
│   │   thematic_analysis.html
|
└───figures
│   │   ASA_chart.png
│   │   thematic_analysis.png


```

| File/Folder name                               | Content                                                                                                                                                                              |
| ---------------------------------------------- | ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------ |
| ChecklistDataRepositoryReview.pdf              | Checklist reviewing the analysis code and data, filled in by Martin Dierikx.                                                                                                         |
| README.md                                      | The file you are currently reading - provides an overview of the repository and how to reproduce our analyses.                                                                       |
| figures/                                       | Folder containing figures created from results of our analysis.                                                                                                                      |
| - ASA_chart.png                                | Figure showing the comparison between Jamie, Siri, and HAL 9000 in terms of their ASAQ scores, Figure 4.8.                                                                           |
| - thematic_analysis.png                        | Figure showing the final codes kept in the thematic analysis, organized in 4 themes: planning, persuasive strategies, virtual coach, and not motivating or demotivating, Figure H.1. |
| results/                                       | Folder containing html files created from the Jupyter notebooks with the results after running them.                                                                                 |
| - analysis_2_3.html                            | Html file obtained by running the corresponding .ipynb file.                                                                                                                         |
| - analysis_4_5_6.html                          | Html file obtained by running the corresponding .ipynb file.                                                                                                                         |
| - asa_questionnaire.html                       | Html file obtained by running the corresponding .ipynb file.                                                                                                                         |
| - bayesian_t_test_confidence.pdf               | Pdf file obtained by knitting the corresponding Rmd file.                                                                                                                            |
| - bayesian_t_test_perceived_usefulness.pdf     | Pdf file obtained by knitting the corresponding Rmd file.                                                                                                                            |
| - bayesian_t_test_start_end.pdf                | Pdf file obtained by knitting the corresponding Rmd file.                                                                                                                            |
| - demographic.html                             | Html file obtained by running the corresponding .ipynb file.                                                                                                                         |
| - effects_of_actions.html                      | Html file obtained by running the corresponding .ipynb file.                                                                                                                         |
| - g_algorithm.html                             | Html file obtained by running the corresponding .ipynb file.                                                                                                                         |
| - planning_habits_and_enjoyment.html           | Html file obtained by running the corresponding .ipynb file.                                                                                                                         |
| - preprocess.html                              | Html file obtained by running the corresponding .ipynb file.                                                                                                                         |
| - thematic_analysis.html                       | Html file obtained by running the corresponding .ipynb file.                                                                                                                         |
| analysis/                                      | Folder containing all the necessary files for the analysis.                                                                                                                          |
| - Dockerfile                                   | The Dockerfile for running the Jupyter notebooks.                                                                                                                                    |
| - demographic.ipynb                            | Jupyter notebook for generating (part of) the table with the demographic data.                                                                                                       |
| - preprocess.ipynb                             | Jupyter notebook for anonymising and preprocessing the data gathered during the study.                                                                                               |
| - main analysis questions/                     | Folder containing the code for the six main questions of the analysis.                                                                                                               |
| - - g_algorithm.ipynb                          | Jupyter notebook for running the G-algorithm to select features, Q1 of the analysis.                                                                                                 |
| - - analysis_2_3.ipynb                         | Jupyter notebook for running the code for predicting rewards and future states, Q2 and Q3 of the analysis.                                                                           |
| - - analysis_4_5_6.ipynb                       | Jupyter notebook for running the code for generating the optimal policy, simulating it, and comparing with two other policies, Q4, Q5, and Q6 of the analysis.                       |
| - - files_for_g/                               | Folder where the G-algorithm places newly created files, which it then deletes once it finishes running.                                                                             |
| - - state_files/                               | Folder containing all the possible states to be used in the model.                                                                                                                   |
| - - - all_possible_states.txt                  | Text file containing all the possible states for two values of confidence and two values of perceived usefulness.                                                                    |
| - - - states_for_g.txt                         | Text file containing all the possible states for 11 values of confidence, 21 values of perceived usefulness, and 21 values of attitude.                                              |
| - - - states_for_q4.txt                        | Text file containing all the possible states for two values of confidence and two values of perceived usefulness, except the end states.                                             |
| - - - states_for_q6.txt                        | Text file containing all the possible states for states without the person's situation, except the end states.                                                                       |
| - exploratory_analysis/                        | Folder containing the code for the exploratory questions of the analysis.                                                                                                            |
| - - asa questionnaire/                         | Folder containing the code for analysing the results of the ASA questionnaire for Jamie, and comparing it with other ASAs.                                                           |
| - - - asa_questionnaire.ipynb                  | Jupyter notebook for running the code for generating Jamie's mean scores in the ASA questionnaire items as well as Siri and HAL 9000's scores for comparison.                        |
| - - effects_of_actions/                        | Folder containing the code for analysing the effects of individual and multiple actions on people's raw states.                                                                      |
| - - - Dockerfile                               | The Dockerfile for running RStudio.                                                                                                                                                  |
| - - - JAGS-4.3.0.tar.gz                        | Compressed archive file necessary for the Bayesian First Aid R package.                                                                                                              |
| - - - effects_of_actions.ipynb                 | Jupyter notebook for splitting the dataset on different criteria and generating files which can be used in the .Rmd files to run Bayesian t-tests.                                   |
| - - - bayesian_t_test_confidence.Rmd           | Rmd file for running a Bayesian t-test for determining the effect of individual actions on confidence.                                                                               |
| - - - bayesian_t_test_perceived_usefulness.Rmd | Rmd file for running a Bayesian t-test for determining the effect of individual actions on perceived usefulness.                                                                     |
| - - - bayesian_t_test_start_end.Rmd            | Rmd file for running a Bayesian t-test for determining the effect of all actions on confidence and perceived usefulness.                                                             |
| - - planning habits and enjoyment/             | Folder containing the code necessary for detemining how often people plan and how much they enjoy it.                                                                                |
| - - - planning_habits_and_enjoyment.ipynb      | Jupyter notebook for calculating how many plans people created on average in the past 3 months and how much they enjoy creating plans.                                               |
| - - thematic analysis/                         | Folder containing the code necessary for the thematic analysis.                                                                                                                      |
| - - - thematic_analysis.ipynb                  | Jupyter notebook for selecting the codes to keep in the thematic analysis, depending on agreement based on Cohen's Kappa and frequency of the codes.                                 |
| data/                                          | The data gathered and needed for the analyses (further explained in the README.md file found in the data folder)                                                                     |

## Reproducing our analyses

### External files required

For the analysis of the answers to the ASA questionnaire, we used the means of 14 ASAs from "Data and analysis underlying the research into the Artificial-Social-Agent Questionnaire: Establishing the long and short questionnaire versions" by Siska Fitrianie, Merijn Bruijnes, Fengxiang Li, Amal Abdulrahman, and Willem-Paul Brinkman, which can be found [here](https://data.4tu.nl/datasets/fd344d58-1381-447c-a167-c6e53eaa0774/5). To run the analysis found in `analysis/exploratory_analysis/asa questionnaire`, please download the `constructsMean_14agents.csv` file and place it in the `data` folder.

To generate the ASA chart, we used the ASA tool from the "Artificial Social Agents Instrument" by, Siska Fitrianie, Merijn Bruijnes, Fengxiang Li, Amal Abdulrahman, and Willem-Paul Brinkman, which can be found [here](https://data.4tu.nl/datasets/d8e2c534-d192-4411-a1e2-5aeac97f0165/6). Normally, the ASA tool generates a chart for up to 2 ASAs, so it was modified to fit three.


### Initial Setup

Make sure that you have Docker installed. You can check whether you do by running `docker -v`. <br>
In case you do not have it installed, you can follow the instructions [here](https://docs.docker.com/get-docker/).

### Obtaining the Docker Image for the Jupyter Notebook Docker Container

You can build the Docker image in the directory of this README file. <br>
Build the Docker image via `docker build . -t <DOCKER_IMAGE_NAME> -f ./analysis/Dockerfile`.

Replace the `<DOCKER_IMAGE_NAME>` with your chosen name of the docker image. For example, if you want to name your docker image `analysis`, your docker command would be `docker build . -t analysis -f ./analysis/Dockerfile`.

### Running the Jupyter Notebook Docker Container

Run the Docker container via `docker run -p 8888:8888 -e JUPYTER_ENABLE_LAB=yes -v <PATH_TO_THIS_DIRECTORY>:/home/jovyan/work <DOCKER_IMAGE_NAME>`. <br>
Replace `<PATH_TO_THIS_DIRECTORY>` with the path to the directory that this README file is in and replace `<DOCKER_IMAGE_NAME>` with the name you chose for the image when creating it.

If you are running a container where the actual path to your current folder is, for instance, `/home/users/documents`, and the name for the docker image you chose was, for example, `analysis`, the command should look something like this: <br>
`docker run -p 8888:8888 -e JUPYTER_ENABLE_LAB=yes -v /home/users/documents:/home/jovyan/work analysis`.

### Obtaining the Docker Image for the R Studio Docker Container

You can build the Docker image in the directory of this README file. <br>
Build the Docker image via `docker build . -t <DOCKER_IMAGE_NAME> -f ./analysis/exploratory_analysis/effects_of_actions/Dockerfile`.

Replace the `<DOCKER_IMAGE_NAME>` with your chosen name of the docker image. For example, if you want to name your docker image `analysis_2`, your docker command would be `docker build . -t analysis_2 -f ./analysis/exploratory_analysis/effects_of_actions/Dockerfile`.

More information regarding `Dockerfile` and how to build images can be found [here](https://docs.docker.com/engine/reference/builder/).

Also, make sure you give your Docker images different names.


### Running the R Studio Docker Container

Run the Docker container via `docker run -d -p 8787:8787 -v <PATH_TO_THIS_DIRECTORY>:/home/rstudio/analysis -e PASSWORD=<PASSWORD> <DOCKER_IMAGE_NAME>`. <br>
Replace `<PATH_TO_THIS_DIRECTORY>` with the path to the directory that this README file is in, choose any password and use it to replace `<PASSWORD>`, and replace `<DOCKER_IMAGE_NAME>` with the name you chose for the image when creating it.

If you are running a container where the actual path to your current folder is, for instance, `/home/users/documents`, and the name for the docker image you chose was, for example, `analysis_2`, and the password you chose is 1234 the command should look something like this: <br>
`docker run -d -p 8787:8787 -v /home/users/documents:/home/rstudio/analysis -e PASSWORD=1234 analysis_2`. <br>

### Running the analyses
    
#### Python

When the container is running, go to one of the links presented in the terminal to access Jupyter Notebook.

Open the ```work``` folder in Jupyter Notebook. <br>
Then, open either the ```analysis/main analysis questions``` folder or the ```analysis/exploratory_analysis``` folder in the Jupyter Notebook.
Note that some of the files in the ```analysis/exporatory_analysis/effects_of_actions``` folder need to be run in R Studio (see below).

Choose the .ipynb file that you want to run and reproduce the analysis.    
    
#### R
    
After running the Docker container, your terminal will display a long number. It does not matter what the number is, as long as it appears. Navigate to http://localhost:8787/ and you will be prompted with a login page. Use rstudio as the username and the password you chose previously (1234 in the example above) as the password.
    
Open the "analysis" folder in R Studio. <br>
Then, open the ```analysis/exporatory_analysis/effects_of_actions``` folder in R Studio.
    
Choose the .Rmd file that you want to run and reproduce the analysis.
    

### Results
- The `results` folder contains the .html files from running the Jupyter Notebook files. These can be obtained by running one of the .ipynb files and downloading them as .html file.


### Tables/Figures and Corresponding Files

| Figure/Table/Result from paper                                                                                | Corresponding analysis file                            | 
|---------------------------------------------------------------------------------------------------------------|--------------------------------------------------------|
| Total number of participants                                                                                  | demographic.ipynb                                      |
| Number of samples                                                                                             | demographic.ipynb                                      |
| Table 4.1                                                                                                     | demographic.ipynb, planning_habits_and_enjoyment.ipynb |
| Number of people who finished the post questionnaire                                                          | demographic.ipynb                                      |
| Figure 4.2                                                                                                    | analysis_2_3.ipynb                                     |         
| Figure 4.3                                                                                                    | analysis_2_3.ipynb                                     |
| Figure 4.4                                                                                                    | analysis_4_5_6.ipynb                                   |
| Percentage of people who end in a good state for the optimal policy                                           | analysis_4_5_6.ipynb                                   |
| Average number of time steps needed to end the conversation for the optimal policy                            | analysis_4_5_6.ipynb                                   |
| Figure 4.5                                                                                                    | analysis_4_5_6.ipynb                                   |
| Figure 4.6                                                                                                    | analysis_4_5_6.ipynb                                   |
| Percentage of people who end in a good state for the worst policy                                             | analysis_4_5_6.ipynb                                   |
| Average number of time steps needed to end the conversation for the worst policy                              | analysis_4_5_6.ipynb                                   |
| Percentage of people who end in a good state for the optimal policy without person's situation                | analysis_4_5_6.ipynb                                   |
| Average number of time steps needed to end the conversation for the optimal policy without person's situation | analysis_4_5_6.ipynb                                   |
| Figure 4.7                                                                                                    | analysis_4_5_6.ipynb                                   |
| Average number of plans created in general                                                                    | planning_habits_and_enjoyment.ipynb                    |
| Average number of plans for walking created                                                                   | planning_habits_and_enjoyment.ipynb                    |
| Enjoyment of creating plans in general                                                                        | planning_habits_and_enjoyment.ipynb                    |
| Enjoyment of creating plans for walking                                                                       | planning_habits_and_enjoyment.ipynb                    |
| ASA score for Jamie                                                                                           | asa_questionnaire.ipynb                                |
| ASA score for Siri                                                                                            | asa_questionnaire.ipynb                                |
| ASA score for HAL 9000                                                                                        | asa_questionnaire.ipynb                                |
| Figure 4.8                                                                                                    | asa_questionnaire.ipynb, ASA_chart.png                 |
| Appendix Table C.1                                                                                            | analysis_2_3.ipynb                                     |
| Appendix Table D.1                                                                                            | g_algorithm.ipynb                                      |
| Appendix Table E.1                                                                                            | analysis_4_5_6.ipynb                                   |
| Appendix Table F.1                                                                                            | analysis_4_5_6.ipynb                                   |
| Appendix Table G.1                                                                                            | analysis_4_5_6.ipynb                                   |
| Appendix Table H.1                                                                                            | bayesian_t_test_confidence.R                           |
| Appendix Table H.2                                                                                            | bayesian_t_test_perceived_usefulness.R                 |
| Appendix Table H.3                                                                                            | bayesian_t_test_start_end.R                            |
| Appendix Table I.2                                                                                            | thematic_analysis.ipynb                                |
| Appendix Table I.3                                                                                            | thematic_analysis.ipynb                                |
| Appendix Figure I.1                                                                                           | thematic_analysis.png                                  |

### Data preprocessing

Since the most of the raw data contained Prolific ids and other fields we were not allowed to publish, we could not publish those files. 
Therefore we only published the preprocessed data files. 
The Python file ```preprocess.ipynb``` was used to preprocess and anonymise the files.
The code therein indicates which fields are kept.
The code can also be run and it will preprocess and anonymise the already processed and anonymised files via the same original steps, generating a new file with the _2 suffix, which can be used for comparison.
